// Demonstrates how to obtain signal strength information from
// some wireless LAN drivers under MS Windows. Compiles under
// Borland Delphi 6.
//
// No support or guaratee of any kind for this piece of software.
// Use at you own risk.
//
// -Vadim

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls;

type
  TOutBuf = array [0..$200B] of Byte;
  TOutBufW = array [0..$1005] of Word;
  POutBufW = ^TOutBufW;
  TForm1 = class(TForm)
    Timer: TTimer;
    ApList: TListView;
    procedure FormCreate(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
  private
    { Private declarations }
    OutBuf: TOutBuf;
    OutBufW: POutBufW;
    InBuf: Cardinal;
    BytesReturned: Cardinal;
    DriverName: string;
    procedure IoControlCall(Handle: THandle; Code, Length: Word);
    procedure IoControlOpenCall(Handle: THandle);
    procedure IoControlScanCall(Handle: THandle);
    procedure IoControlCloseCall(Handle: THandle);
    procedure GetApInfo;
    procedure PrintResult;
    procedure FindDriverName;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FindDriverName;
var
  Key: HKey;
  Buf: array [0..$FFFF] of Char;
  ByteCount: Cardinal;
  KeyType: Cardinal;
begin
  DriverName:='';
  if RegOpenKey(HKEY_LOCAL_MACHINE, 'SYSTEM\CurrentControlSet\Enum\PCMCIA\Lucent_Technologies-WaveLAN/IEEE-C908\1', Key) <> ERROR_SUCCESS then
  begin
    ShowMessage('Can''t find WaveLan card');
    Close;
  end;
  try
    ByteCount:=SizeOf(Buf);
    if RegQueryValueEx(Key, 'Driver', nil, @KeyType, @Buf, @ByteCount) <> ERROR_SUCCESS then
    begin
      ShowMessage('Can''t find WaveLan card driver');
      Close;
    end;
  finally
    RegCloseKey(Key);
  end;
  if RegOpenKey(HKEY_LOCAL_MACHINE, PChar('SYSTEM\CurrentControlSet\Control\Class\'+Buf), Key) <> ERROR_SUCCESS then
  begin
    ShowMessage('Can''t find WaveLan card driver name');
    Close;
  end;
  try
    ByteCount:=SizeOf(Buf);
    if RegQueryValueEx(Key, 'NetCfgInstanceId', nil, @KeyType, @Buf, @ByteCount) <> ERROR_SUCCESS then
    begin
      ShowMessage('Can''t load WaveLan driver name');
      Close;
    end;
    DriverName:='\\.\'+Buf;
  finally
    RegCloseKey(Key);
  end;
  if DriverName='' then
  begin
    ShowMessage('Error finding WaveLan driver');
    Close;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  OutBufW:=Addr(OutBuf);
  InBuf:=$FF020401;
  FindDriverName;
end;

procedure TForm1.IoControlCall(Handle: THandle; Code, Length: Word);
begin
  OutBufW[2]:=0;
  OutBufW[3]:=4;
  OutBufW[4]:=Length;
  OutBufW[5]:=Code;
  if not DeviceIoControl(Handle, $170002, @InBuf, SizeOf(InBuf), @OutBuf, SizeOf(OutBuf), BytesReturned, nil)
  then ShowMessageFmt('DeviceIoControl Error [%d]', [GetLastError]);
end;

procedure TForm1.IoControlScanCall(Handle: THandle);
begin
  OutBufW[2]:=0;
  OutBufW[3]:=2;
  OutBufW[4]:=$800;
  OutBufW[5]:=6;
  if not DeviceIoControl(Handle, $170002, @InBuf, SizeOf(InBuf), @OutBuf, SizeOf(OutBuf), BytesReturned, nil)
  then ShowMessageFmt('DeviceIoControl Error [%d]', [GetLastError]);
end;

procedure TForm1.IoControlCloseCall(Handle: THandle);
begin
  OutBufW[2]:=$45;
  OutBufW[3]:=1;
  OutBufW[4]:=0;
  OutBufW[5]:=0;
  if not DeviceIoControl(Handle, $170002, @InBuf, SizeOf(InBuf), @OutBuf, SizeOf(OutBuf), BytesReturned, nil)
  then ShowMessageFmt('DeviceIoControl Error [%d]', [GetLastError]);
end;

procedure TForm1.IoControlOpenCall(Handle: THandle);
begin
  if not DeviceIoControl(Handle, $170002, @InBuf, SizeOf(InBuf), @OutBuf, SizeOf(OutBuf), BytesReturned, nil)
  then ShowMessageFmt('DeviceIoControl Error [%d]', [GetLastError]);
end;

procedure TForm1.PrintResult;
var
  i, j, k: Integer;
  APCount: Integer;
  Item: TListItem;
  MacString: string;
begin
  APCount:=(BytesReturned-12) div 50;
  if APCount=0 then Exit;
  for i:=0 to APCount-1 do
  begin
    j:=6+i*25;
    MacString:=Format('%.4X%.4X%.4X', [Swap(OutBufW[j+3]), Swap(OutBufW[j+4]), Swap(OutBufW[j+5])]);
    Item:=nil;
    for k:=0 to ApList.Items.Count-1 do
    if ApList.Items[k].Caption=MacString then Item:=ApList.Items[k];
    if Item=nil then
    begin
      Item:=ApList.Items.Add();
      Item.SubItems.Add('');
      Item.SubItems.Add('');
      Item.SubItems.Add('');
      Item.SubItems.Add('');
    end;
    Item.Caption:=MacString; // MAC address of access point
    Item.SubItems[0]:=Format('%d',[OutBufW[j]]); // Channel
    Item.SubItems[1]:=Format('%d',[OutBufW[j+2]]); // Signal
    Item.SubItems[2]:=Format('%d',[OutBufW[j+1]]); // Noise
    Item.SubItems[3]:=PChar(@OutBuf[j*2+18]); // Net ID
  end;
end;

procedure TForm1.GetApInfo;
var
  Handle: THandle;
  Str: string;
begin
  // Open WaveLan driver for DeviceIoControl access
  Handle:=
  CreateFile(PChar(DriverName),
             0,
             FILE_SHARE_READ or FILE_SHARE_WRITE,
             nil,
             OPEN_EXISTING,
             0,
             0);
  if Handle=INVALID_HANDLE_VALUE then
  begin
    // Some error. For example,
    // the driver may already be opened by another application
    ShowMessageFmt('Can''t open the driver [%d]', [GetLastError]);
    Exit;
  end;
  try
    FillChar(OutBuf, SizeOf(OutBuf), 0);
    IoControlOpenCall(Handle);
    if (OutBufW[0]<>0) or (OutBufW[1]<>0) then
    // Driver ready for action
    try
      repeat
        // Read access point scan results
        IoControlCall(Handle, $820, $5DD);
        PrintResult;
      // BytesReturned in the last IoConrolCall
      until (BytesReturned<=12) or (BytesReturned>$5DD);
      // Broadcast the scan request
      IoControlScanCall(Handle);
    finally
      // Tell the driver we are done.
      IoControlCloseCall(Handle);
    end;
  finally
    CloseHandle(Handle);
  end;
end;

// Function called periodically (i.e. every 250 ms) by timer.
procedure TForm1.TimerTimer(Sender: TObject);
begin
  GetApInfo;
end;

end.
